module lfsr_gen #(
    parameter LFSR_LENGTH = 8,
    parameter SEED = 1
)(
    input wire clk,
    input wire ena, // enables output to change
    output reg [LFSR_LENGTH-1:0] dout
);

reg [LFSR_LENGTH-1:0] sh_reg = SEED;
wire feedback;

// Function to calculate feedback bit
function automatic getFeedback;
    input [56:0] shreg;
    begin
        case (LFSR_LENGTH)
            // Specify feedback logic for different LFSR lengths
            3:  getFeedback = shreg[3] ^ shreg[2];
            4:  getFeedback = shreg[4] ^ shreg[3];
            5:  getFeedback = shreg[5] ^ shreg[3];
            6:  getFeedback = shreg[6] ^ shreg[5];
            7:  getFeedback = shreg[7] ^ shreg[6];
            8:  getFeedback = shreg[8] ^ shreg[6] ^ shreg[5] ^ shreg[4];
            9:  getFeedback = shreg[9] ^ shreg[5];
            10: getFeedback = shreg[10] ^ shreg[7];
            11: getFeedback = shreg[11] ^ shreg[9];
            12: getFeedback = shreg[12] ^ shreg[6] ^ shreg[4] ^ shreg[1];
            13: getFeedback = shreg[13] ^ shreg[4] ^ shreg[3] ^ shreg[1];
            14: getFeedback = shreg[14] ^ shreg[5] ^ shreg[3] ^ shreg[1];
            15: getFeedback = shreg[15] ^ shreg[14];
            16: getFeedback = shreg[16] ^ shreg[15] ^ shreg[13] ^ shreg[4];
            17: getFeedback = shreg[17] ^ shreg[14];
            18: getFeedback = shreg[18] ^ shreg[11];
            19: getFeedback = shreg[19] ^ shreg[6] ^ shreg[2] ^ shreg[1];
            20: getFeedback = shreg[20] ^ shreg[17];
            21: getFeedback = shreg[21] ^ shreg[19];
            22: getFeedback = shreg[22] ^ shreg[21];
            23: getFeedback = shreg[23] ^ shreg[18];
            24: getFeedback = shreg[24] ^ shreg[23] ^ shreg[22] ^ shreg[17];
            25: getFeedback = shreg[25] ^ shreg[22];
            26: getFeedback = shreg[26] ^ shreg[6] ^ shreg[2] ^ shreg[1];
            27: getFeedback = shreg[27] ^ shreg[5] ^ shreg[2] ^ shreg[1];
            28: getFeedback = shreg[28] ^ shreg[25];
            29: getFeedback = shreg[29] ^ shreg[27];
            30: getFeedback = shreg[30] ^ shreg[6] ^ shreg[4] ^ shreg[1];
            31: getFeedback = shreg[31] ^ shreg[28];
            32: getFeedback = shreg[32] ^ shreg[22] ^ shreg[2] ^ shreg[1];
            33: getFeedback = shreg[33] ^ shreg[20];
            34: getFeedback = shreg[34] ^ shreg[27] ^ shreg[2] ^ shreg[1];
            35: getFeedback = shreg[35] ^ shreg[33];
            36: getFeedback = shreg[36] ^ shreg[25];
            37: getFeedback = shreg[37] ^ shreg[5] ^ shreg[4] ^ shreg[3] ^ shreg[2] ^ shreg[1];
            38: getFeedback = shreg[38] ^ shreg[6] ^ shreg[5] ^ shreg[1];
            39: getFeedback = shreg[39] ^ shreg[35];
            40: getFeedback = shreg[40] ^ shreg[38] ^ shreg[21] ^ shreg[19];
            41: getFeedback = shreg[41] ^ shreg[38];
            42: getFeedback = shreg[42] ^ shreg[41] ^ shreg[20] ^ shreg[19];
            43: getFeedback = shreg[43] ^ shreg[42] ^ shreg[38] ^ shreg[37];
            44: getFeedback = shreg[44] ^ shreg[43] ^ shreg[18] ^ shreg[17];
            45: getFeedback = shreg[45] ^ shreg[44] ^ shreg[42] ^ shreg[41];
            46: getFeedback = shreg[46] ^ shreg[45] ^ shreg[26] ^ shreg[25];
            47: getFeedback = shreg[47] ^ shreg[42];
            48: getFeedback = shreg[48] ^ shreg[47] ^ shreg[21] ^ shreg[20];
            49: getFeedback = shreg[49] ^ shreg[40];
            50: getFeedback = shreg[50] ^ shreg[49] ^ shreg[24] ^ shreg[23];
            51: getFeedback = shreg[51] ^ shreg[50] ^ shreg[36] ^ shreg[35];
            52: getFeedback = shreg[52] ^ shreg[49];
            53: getFeedback = shreg[53] ^ shreg[52] ^ shreg[38] ^ shreg[37];
            54: getFeedback = shreg[54] ^ shreg[53] ^ shreg[18] ^ shreg[17];
            55: getFeedback = shreg[55] ^ shreg[31];
            56: getFeedback = shreg[56] ^ shreg[55] ^ shreg[35] ^ shreg[34];

            // Continue for all defined LFSR lengths in VHDL code
            default: begin
                // Handle unsupported LFSR lengths
                getFeedback = 1'bx; // Undefined behavior
                $display("LFSR register length out of defined range");
            end
        endcase
    end
endfunction

// Shift register process
always @(posedge clk) begin
    sh_reg <= {sh_reg[LFSR_LENGTH-2:0], feedback};
end

// Calculate feedback using the function
assign feedback = getFeedback(sh_reg);

// Output enable process
always @(posedge clk) begin
    if (ena) begin
        dout <= sh_reg;
    end
end

endmodule
