module w02_pipelining(
    input clk,				// clock 50 MHz (PIN_AF14)
    output [35:0] dout
);

localparam LFSR_LENGTH = 34;


//-----------------------------------------------------------------------------
// ena generator
//-----------------------------------------------------------------------------
// generate ena = 1 every 8 clk cycles
reg ena;
reg [2:0] ena_cnt;
initial begin
    ena = 1'b0;
    ena_cnt = 3'b000;
end
always @(posedge clk) begin
    ena_cnt <= ena_cnt + 1;
    if (ena_cnt == 3'b111)
        ena <= 1'b1;
    else
        ena <= 1'b0;
end

//-----------------------------------------------------------------------------
// instantiate PRN generators
//-----------------------------------------------------------------------------
wire [33:0] a,b,c,d;

lfsr_gen #(
    .LFSR_LENGTH(LFSR_LENGTH),
    .SEED(123123123)
) lfsr_gen_A(
    .clk(clk),
    .ena(ena),
    .dout(a)
);

lfsr_gen #(
    .LFSR_LENGTH(LFSR_LENGTH),
    .SEED(123123124)
) lfsr_gen_B(
    .clk(clk),
    .ena(ena),
    .dout(b)
);

lfsr_gen #(
    .LFSR_LENGTH(LFSR_LENGTH),
    .SEED(123123125)
) lfsr_gen_C(
    .clk(clk),
    .ena(ena),
    .dout(c)
);

lfsr_gen #(
    .LFSR_LENGTH(LFSR_LENGTH),
    .SEED(123123126)
) lfsr_gen_D(
    .clk(clk),
    .ena(ena),
    .dout(d)
);

//-----------------------------------------------------------------------------
// calculate results
//-----------------------------------------------------------------------------
reg [35:0] y;
reg [34:0] m,n;
always @(posedge clk) begin
    y <= a + b + c + d;
end


// assign the result to the output
assign dout = y;  

    
endmodule
